package com.me.pacman.entity;

import com.me.pacman.level.LevelComponent;
import com.me.pacman.state.LevelState;

public abstract class MovableEntity extends Entity {

    // speed in tiles / second.
    public float speed;
    public boolean moving;
    public Direction currDirection;
    private float tolerance;

    public Direction nextDirection = null;
    public boolean canMove = true;

    public MovableEntity(LevelState state, float x, float y, float speed, boolean moving, Direction currDirection, float turnTolerance) {
        super(state, x, y);
        this.speed = speed;
        this.moving = moving;
        this.currDirection = currDirection;
        this.tolerance = turnTolerance;
    }

    public void update(float dt) {
        super.update(dt);

        if (!moving) {
            return;
        }

        if (nextDirection != null) {
            LevelComponent nextComponent;
            boolean turned = false;
            switch (nextDirection) {
                case NORTH:
                    nextComponent = state.level.getComponent(x, y + 1f);
                    if (nextComponent.isPassable() && Math.abs(x - ((int) x + 0.5f)) <= tolerance) {
                        x = ((int) x) + 0.5f;
                        turned = true;
                    }
                    break;
                case EAST:
                    nextComponent = state.level.getComponent(x + 1f, y);
                    if (nextComponent.isPassable() && Math.abs(y - ((int) y + 0.5f)) <= tolerance) {
                        y = ((int) y) + 0.5f;
                        turned = true;
                    }
                    break;
                case SOUTH:
                    nextComponent = state.level.getComponent(x, y - 1f);
                    if (nextComponent.isPassable() && Math.abs(x - ((int) x + 0.5f)) <= tolerance) {
                        x = ((int) x) + 0.5f;
                        turned = true;
                    }
                    break;
                case WEST:
                    nextComponent = state.level.getComponent(x - 1f, y);
                    if (nextComponent.isPassable() && Math.abs(y - ((int) y + 0.5f)) <= tolerance) {
                        y = ((int) y) + 0.5f;
                        turned = true;
                    }
                    break;
            }
            if (turned) {
                currDirection = nextDirection;
                nextDirection = null;
            }
        }

        float dist = speed * dt;
        float new_y = y;
        float new_x = x;

        switch (currDirection) {
            case NORTH:
                new_y += dist;
                canMove = state.level.getComponent(new_x, new_y + 0.5f).isPassable();
                break;
            case EAST:
                new_x += dist;
                canMove = state.level.getComponent(new_x + 0.5f, new_y).isPassable();
                break;
            case SOUTH:
                new_y -= dist;
                canMove = state.level.getComponent(new_x, new_y - 0.5f).isPassable();
                break;
            case WEST:
                new_x -= dist;
                canMove = state.level.getComponent(new_x - 0.5f, new_y).isPassable();
                break;
        }

        // if move wasn't going to collide with wall, move normally.
        // otherwise, trim would-be decimal and center entity on tile
        x = canMove? new_x : ((int) new_x) + 0.5f;
        y = canMove? new_y : ((int) new_y) + 0.5f;
    }

    public enum Direction {
        NORTH,
        EAST,
        SOUTH,
        WEST
    }

}
